import itertools
import os
from typing import List, Optional, Union

import numpy as np
import pandas as pd

from simba.data_processors.timebins_movement_calculator import \
    TimeBinsMovementCalculator
from simba.mixins.config_reader import ConfigReader
from simba.mixins.feature_extraction_supplement_mixin import \
    FeatureExtractionSupplemental
from simba.roi_tools.roi_aggregate_statistics_analyzer import \
    ROIAggregateStatisticsAnalyzer
from simba.utils.checks import (
    check_all_file_names_are_represented_in_video_log,
    check_file_exist_and_readable, check_float, check_if_dir_exists,
    check_valid_boolean, check_valid_lst)
from simba.utils.data import detect_bouts, slice_roi_dict_for_video
from simba.utils.errors import (CountError, FrameRangeError,
                                ROICoordinatesNotFoundError)
from simba.utils.printing import SimbaTimer, stdout_success
from simba.utils.read_write import get_fn_ext, read_data_paths, read_df
from simba.utils.warnings import ROIWarning

SHAPE_TYPE = "Shape_type"
TOTAL_ROI_TIME = 'TOTAL ROI TIME (S)'
ENTRY_COUNTS = 'ROI ENTRIES (COUNTS)'
FIRST_ROI_ENTRY_TIME = 'FIRST ROI ENTRY TIME (S)'
LAST_ROI_ENTRY_TIME = 'LAST ROI ENTRY TIME (S)'
MEAN_BOUT_TIME = 'MEAN ROI BOUT TIME (S)'
VELOCITY = 'AVERAGE ROI VELOCITY (CM/S)'
MOVEMENT = 'TOTAL ROI MOVEMENT (CM)'
MEASUREMENT = 'MEASUREMENT'
VIDEO_FPS = 'VIDEO FPS'
VIDEO_LENGTH = 'VIDEO LENGTH (S)'
PIX_PER_MM = 'PIXEL TO MILLIMETER CONVERSION FACTOR'
OUTSIDE_ROI = 'OUTSIDE REGIONS OF INTEREST'


class ROITimebinAnalyzer(ConfigReader):
    """
    Analyzes region-of-interest (ROI) data from video tracking experiments conditioned on time-bin.

    This class computes various statistics related to body-part movements inside defined ROIs,
    including entry counts, total time spent, and bout durations.

    :param config_path (str | os.PathLike): Path to the configuration file.
    :param data_path (str | os.PathLike | List[str], optional): Path(s) to the data files.
    :param threshold (float): Probability threshold for body-part inclusion.
    :param body_parts (List[str], optional): List of body parts to analyze.
    :param detailed_bout_data (bool): Whether to compute detailed bout data.
    :param calculate_distances (bool): Whether to compute distances traveled.
    :param total_time (bool): Whether to calculate total time spent in ROIs.
    :param entry_counts (bool): Whether to count entries into ROIs.
    :param first_entry_time (bool): Whether to record the first entry time.
    :param include_time_stamps (bool): Whether to include time bin start times, start frame, end time and end frame in output.
    :param outside_rois (bool): If checked, SimBA will treat all areas NOT covered by a ROI drawing as a single additional ROI and compute the chosen metrics for this, single, ROI.
    :param last_entry_time (bool): Whether to record the last entry time.
    :param transpose (bool): Whether to transpose the final results.
    :param detailed_bout_data_save_path (str | os.PathLike, optional): Path to save detailed bout data.
    :param save_path (str | os.PathLike, optional): Path to save summary statistics.

    .. note::

    :example:
    >>> test = ROITimebinAnalyzer(config_path=r"C:\troubleshooting\mitra\project_folder\project_config.ini", bin_size=61, body_parts=['Nose'], detailed_bout_data=True, calculate_distances=True, transpose=True)
    >>> test.run()
    >>> test.save()

    """

    def __init__(self,
                 config_path: Union[str, os.PathLike],
                 bin_size: float,
                 data_path: Optional[Union[str, os.PathLike, List[str]]] = None,
                 threshold: float = 0.0,
                 body_parts: Optional[List[str]] = None,
                 detailed_bout_data: bool = False,
                 calculate_distances: bool = False,
                 total_time: bool = True,
                 entry_counts: bool = True,
                 first_entry_time: bool = False,
                 last_entry_time: bool = False,
                 transpose: bool = False,
                 outside_rois: bool = False,
                 verbose: bool = True,
                 include_fps: bool = False,
                 include_video_length: bool = False,
                 include_px_per_mm: bool = False,
                 include_time_stamps: bool = False,
                 detailed_bout_data_save_path: Optional[Union[str, os.PathLike]] = None,
                 save_path: Optional[Union[str, os.PathLike]] = None):

        check_file_exist_and_readable(file_path=config_path)
        ConfigReader.__init__(self, config_path=config_path)
        if not os.path.isfile(self.roi_coordinates_path):
            raise ROICoordinatesNotFoundError(expected_file_path=self.roi_coordinates_path)
        check_valid_lst(data=body_parts, source=f"{self.__class__.__name__} body-parts", valid_dtypes=(str,), valid_values=self.project_bps)
        check_float(name="Body-part probability threshold", value=threshold, min_value=0.0, max_value=1.0)
        check_float(name="bin_size", value=bin_size, min_value=10e-6)
        if len(set(body_parts)) != len(body_parts):
            raise CountError(msg=f"All body-part entries have to be unique. Got {body_parts}", source=self.__class__.__name__)
        if detailed_bout_data_save_path is not None:
            check_if_dir_exists(in_dir=os.path.dirname(detailed_bout_data_save_path))
        else:
            detailed_bout_data_save_path = os.path.join(self.logs_path, f'{"Detailed_ROI_data"}_{self.datetime}.csv')
        if save_path is not None:
            check_if_dir_exists(in_dir=os.path.dirname(save_path))
        else:
            save_path = os.path.join(self.logs_path, f"ROI_time_bins_{bin_size}s_data_{self.datetime}.csv")
        self.detailed_bout_data_save_path, self.save_path = detailed_bout_data_save_path, save_path
        check_valid_boolean(value=[detailed_bout_data], source=f'{self.__class__.__name__} detailed_bout_data', raise_error=True)
        check_valid_boolean(value=[total_time], source=f'{self.__class__.__name__} total_time', raise_error=True)
        check_valid_boolean(value=[entry_counts], source=f'{self.__class__.__name__} entry_counts', raise_error=True)
        check_valid_boolean(value=[first_entry_time], source=f'{self.__class__.__name__} first_entry_time', raise_error=True)
        check_valid_boolean(value=[last_entry_time], source=f'{self.__class__.__name__} last_entry_time', raise_error=True)
        check_valid_boolean(value=[calculate_distances], source=f'{self.__class__.__name__} calculate_distances', raise_error=True)
        check_valid_boolean(value=[transpose], source=f'{self.__class__.__name__} transpose', raise_error=True)
        check_valid_boolean(value=[include_fps], source=f'{self.__class__.__name__} include_fps', raise_error=True)
        check_valid_boolean(value=[include_video_length], source=f'{self.__class__.__name__} include_video_length', raise_error=True)
        check_valid_boolean(value=[include_px_per_mm], source=f'{self.__class__.__name__} include_px_per_mm', raise_error=True)
        check_valid_boolean(value=[include_time_stamps], source=f'{self.__class__.__name__} include_time_stamps', raise_error=True)
        check_valid_boolean(value=[outside_rois], source=f'{self.__class__.__name__} outside_rois', raise_error=True)
        check_valid_boolean(value=[verbose], source=f'{self.__class__.__name__} verbose', raise_error=True)
        self.read_roi_data()
        self.data_paths = read_data_paths(path=data_path, default=self.outlier_corrected_paths, default_name=self.outlier_corrected_dir, file_type=self.file_type)
        self.bp_dict, self.bp_lk = {}, {}
        for bp in body_parts:
            animal = self.find_animal_name_from_body_part_name(bp_name=bp, bp_dict=self.animal_bp_dict)
            self.bp_dict[animal] = [f'{bp}_{"x"}', f'{bp}_{"y"}', f'{bp}_{"p"}']
            self.bp_lk[animal] = bp
        self.roi_headers = [v for k, v in self.bp_dict.items()]
        self.roi_headers = [item for sublist in self.roi_headers for item in sublist]
        self.calculate_distances, self.threshold = calculate_distances, threshold
        self.detailed_bout_data = detailed_bout_data
        self.total_time, self.entry_counts, self.body_parts, self.include_time_stamps = total_time, entry_counts, body_parts, include_time_stamps
        self.first_entry_time, self.last_entry_time, self.include_px_per_mm = first_entry_time, last_entry_time, include_px_per_mm
        self.transpose, self.include_fps, self.include_video_length, self.bin_size, self.non_roi_zone = transpose, include_fps, include_video_length, bin_size, outside_rois
        self.detailed_dfs, self.detailed_df, self.verbose = [], [], verbose
        self.results = pd.DataFrame(columns=["VIDEO", "SHAPE", "ANIMAL", "BODY-PART", "TIME-BIN #", 'TIME-BIN START TIME (S)', 'TIME-BIN END TIME (S)', 'TIME-BIN START FRAME (#)', 'TIME-BIN END FRAME (#)', "MEASUREMENT", 'VALUE'])
        self.movement_timebins = TimeBinsMovementCalculator(config_path=config_path, bin_length=bin_size, body_parts=body_parts, plots=False, verbose=self.verbose)
        self.movement_timebins.run()

    def __clean_results(self):
        if not self.total_time:
            self.results = self.results[self.results[MEASUREMENT] != TOTAL_ROI_TIME]
        if not self.entry_counts:
            self.results = self.results[self.results[MEASUREMENT] != ENTRY_COUNTS]
        if not self.first_entry_time:
            self.results = self.results[self.results[MEASUREMENT] != FIRST_ROI_ENTRY_TIME]
        if not self.last_entry_time:
            self.results = self.results[self.results[MEASUREMENT] != LAST_ROI_ENTRY_TIME]
        if not self.calculate_distances:
            self.results = self.results[self.results[MEASUREMENT] != VELOCITY]
            self.results = self.results[self.results[MEASUREMENT] != MOVEMENT]
        if not self.include_fps:
            self.results = self.results[self.results[MEASUREMENT] != VIDEO_FPS]
        if not self.include_video_length:
            self.results = self.results[self.results[MEASUREMENT] != VIDEO_LENGTH]
        if not self.include_px_per_mm:
            self.results = self.results[self.results[MEASUREMENT] != PIX_PER_MM]
        if not self.include_time_stamps:
            self.results = self.results.drop(['TIME-BIN START TIME (S)', 'TIME-BIN END TIME (S)', 'TIME-BIN START FRAME (#)', 'TIME-BIN END FRAME (#)'], axis=1)
        self.results = self.results.sort_values(by=["VIDEO", "ANIMAL", "BODY-PART", "SHAPE", "TIME-BIN #", "MEASUREMENT"]).reset_index(drop=True)
        if self.transpose:
            self.results['VALUE'] = pd.to_numeric(self.results['VALUE'], errors='coerce')
            if not self.include_time_stamps:
                self.results = self.results.pivot_table(index=["VIDEO"], columns=["ANIMAL", "SHAPE", "TIME-BIN #", "MEASUREMENT"], values="VALUE")
            else:
                self.results = self.results.pivot_table(index=["VIDEO"], columns=["ANIMAL", "SHAPE", "TIME-BIN #", 'TIME-BIN START TIME (S)', 'TIME-BIN END TIME (S)', 'TIME-BIN START FRAME (#)', 'TIME-BIN END FRAME (#)',  "MEASUREMENT"], values="VALUE")
            self.results = self.results.fillna(value='None')
        else:
            self.results = self.results.set_index('VIDEO')

        if self.detailed_bout_data and (len(self.detailed_df) > 0):
            self.detailed_df = self.detailed_df.rename(columns={"Event": "SHAPE NAME", "Start_time": "START TIME", "End Time": "END TIME", "Start_frame": "START FRAME", "End_frame": "END FRAME", "Bout_time": "DURATION (S)"})
            self.detailed_df["BODY-PART"] = self.detailed_df["ANIMAL"].map(self.bp_lk)
            self.detailed_df = self.detailed_df[["VIDEO", "ANIMAL", "BODY-PART", "SHAPE NAME", "START TIME", "END TIME", "START FRAME", "END FRAME", "DURATION (S)"]].reset_index(drop=True)

    def run(self):
        check_all_file_names_are_represented_in_video_log(video_info_df=self.video_info_df, data_paths=self.data_paths)
        roi_analyzer = ROIAggregateStatisticsAnalyzer(config_path=self.config_path,
                                                      data_path=self.data_paths,
                                                      threshold=self.threshold,
                                                      body_parts=self.body_parts,
                                                      detailed_bout_data=True,
                                                      verbose=self.verbose)
        roi_analyzer.run()
        self.detailed_df = pd.concat(roi_analyzer.detailed_dfs, axis=0)
        for file_cnt, file_path in enumerate(self.data_paths):
            video_timer = SimbaTimer(start=True)
            _, video_name, _ = get_fn_ext(filepath=file_path)
            if self.verbose: print(f"Analysing ROI data for video {video_name}... (Video {file_cnt + 1}/{len(self.data_paths)})")
            _, px_per_mm, fps = self.read_video_info(video_name=video_name)
            video_df = self.detailed_df[self.detailed_df['VIDEO'] == video_name].reset_index(drop=True)
            _, video_shape_names = slice_roi_dict_for_video(data=self.roi_dict, video_name=video_name)
            if len(video_shape_names) == 0:
                ROIWarning(msg=f'Video {video_name} has no drawn ROIs. Skipping video ROI time-bin analysis.', source=self.__class__.__name__)
            frames_per_bin = int(fps * self.bin_size)
            if frames_per_bin == 0:
                raise FrameRangeError(msg=f"The specified time-bin length of {self.bin_size} is TOO SHORT for video {video_name} which has a specified FPS of {fps}. This combination produces time bins that are LESS THAN a single frame.", source=self.__class__.__name__)
            video_frms = list(range(0, len(read_df(file_path=file_path, file_type=self.file_type))))
            video_length = len(video_frms) / fps
            frame_bins = [video_frms[i: i + (frames_per_bin)] for i in range(0, len(video_frms), frames_per_bin)]
            for animal_name, roi_name in list(itertools.product(self.bp_dict.keys(), video_shape_names)):
                bp_name = self.bp_lk[animal_name]
                video_roi_animal_df = video_df.loc[(video_df["Event"] == roi_name) & (video_df["ANIMAL"] == animal_name)]
                entry_frms = list(video_roi_animal_df['Start_frame'])
                inside_roi_frm_idx = [list(range(x, y)) for x, y in zip(list(video_roi_animal_df['Start_frame'].astype(int)), list(video_roi_animal_df["End_frame"].astype(int) + 1))]
                inside_roi_frm_idx = [i for s in inside_roi_frm_idx for i in s]
                for bin_cnt, bin_frms in enumerate(frame_bins):
                    if self.verbose: print(f"Analysing time-bin {bin_cnt+1} (bin: {bin_cnt+1}/{len(frame_bins)}, roi: {roi_name}, animal: {animal_name}, video: {file_cnt + 1}/{len(self.data_paths)}, video name: {video_name})")
                    bin_start_time, bin_end_time = bin_frms[0] / fps, bin_frms[-1] / fps
                    bin_start_frm, bin_end_frm = bin_frms[0], bin_frms[-1]
                    frms_inside_roi_in_timebin_idx = [x for x in inside_roi_frm_idx if x in bin_frms]
                    entry_roi_in_timebin_idx = [x for x in entry_frms if x in bin_frms]
                    if len(frms_inside_roi_in_timebin_idx) > 0:
                        total_time = len(frms_inside_roi_in_timebin_idx) / fps
                        bin_move = (self.movement_timebins.movement_dict[video_name].iloc[frms_inside_roi_in_timebin_idx].values.flatten().astype(np.float32))
                        distance, velocity = FeatureExtractionSupplemental.distance_and_velocity(x=bin_move, fps=fps, pixels_per_mm=1, centimeters=False)
                    else:
                        total_time = 0
                        distance, velocity = 0, None
                    if len(entry_roi_in_timebin_idx) > 0:
                        entry_counts = len(entry_roi_in_timebin_idx)
                        first_entry_time = entry_roi_in_timebin_idx[0] / fps
                        last_entry_time = entry_roi_in_timebin_idx[-1] / fps
                    else:
                        entry_counts = 0
                        first_entry_time = None
                        last_entry_time = None

                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, TOTAL_ROI_TIME, total_time]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, ENTRY_COUNTS, entry_counts]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, FIRST_ROI_ENTRY_TIME, first_entry_time]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, LAST_ROI_ENTRY_TIME, last_entry_time]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, MOVEMENT,  distance]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VELOCITY, velocity]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VIDEO_FPS, fps]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VIDEO_LENGTH, video_length]
                    self.results.loc[len(self.results)] = [video_name, roi_name, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, PIX_PER_MM, px_per_mm]

            if self.non_roi_zone:
                for animal_name in self.bp_dict.keys():
                    video_animal_df = video_df.loc[video_df["ANIMAL"] == animal_name]
                    video_animal_df = video_animal_df.rename(columns={"Event": "SHAPE NAME", "Start_time": "START TIME", "End Time": "END TIME", "Start_frame": "START FRAME", "End_frame": "END FRAME", "Bout_time": "DURATION (S)"})
                    inside_rois_frm = [i for s, e in zip(video_animal_df['START FRAME'], video_animal_df['END FRAME'])  for i in range(s, e + 1)]
                    outside_rois_frm = pd.DataFrame(data=[1 if i not in inside_rois_frm else 0 for i in range(len(video_frms))], columns=[OUTSIDE_ROI])
                    outside_roi_bouts = detect_bouts(data_df=outside_rois_frm, target_lst=OUTSIDE_ROI, fps=fps)
                    outside_roi_frm_idx = [list(range(x, y)) for x, y in zip(list(outside_roi_bouts['Start_frame'].astype(int)), list(outside_roi_bouts["End_frame"].astype(int) + 1))]
                    outside_roi_frm_idx = [i for s in outside_roi_frm_idx for i in s]


                    for bin_cnt, bin_frms in enumerate(frame_bins):
                        if self.verbose: print(f"Analysing OUTSIDE ROI time-bin {bin_cnt + 1} (bin: {bin_cnt + 1}/{len(frame_bins)}, roi: {OUTSIDE_ROI}, animal: {animal_name}, video: {file_cnt + 1}/{len(self.data_paths)}, video name: {video_name})")
                        bin_start_time, bin_end_time = bin_frms[0] / fps, bin_frms[-1] / fps
                        bin_start_frm, bin_end_frm = bin_frms[0], bin_frms[-1]
                        frms_outside_roi_in_timebin_idx = [x for x in outside_roi_frm_idx if x in bin_frms]
                        entry_roi_in_timebin_idx = [x for x in entry_frms if x in bin_frms]
                        if len(frms_outside_roi_in_timebin_idx) > 0:
                            total_time = len(frms_outside_roi_in_timebin_idx) / fps
                            bin_move = (self.movement_timebins.movement_dict[video_name].iloc[frms_outside_roi_in_timebin_idx].values.flatten().astype(np.float32))
                            distance, velocity = FeatureExtractionSupplemental.distance_and_velocity(x=bin_move, fps=fps, pixels_per_mm=1, centimeters=False)
                        else:
                            total_time = 0
                            distance, velocity = 0, None
                        if len(entry_roi_in_timebin_idx) > 0:
                            entry_counts = len(entry_roi_in_timebin_idx)
                            first_entry_time = entry_roi_in_timebin_idx[0] / fps
                            last_entry_time = entry_roi_in_timebin_idx[-1] / fps
                        else:
                            entry_counts = 0
                            first_entry_time = None
                            last_entry_time = None

                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, TOTAL_ROI_TIME, total_time]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, ENTRY_COUNTS, entry_counts]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, FIRST_ROI_ENTRY_TIME, first_entry_time]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, LAST_ROI_ENTRY_TIME, last_entry_time]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, MOVEMENT,  distance]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VELOCITY, velocity]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VIDEO_FPS, fps]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, VIDEO_LENGTH, video_length]
                        self.results.loc[len(self.results)] = [video_name, OUTSIDE_ROI, animal_name, bp_name, bin_cnt, bin_start_time, bin_end_time, bin_start_frm, bin_end_frm, PIX_PER_MM, px_per_mm]

            video_timer.stop_timer()
            if self.verbose: print(f'ROI analysis video {video_name} complete... (elapsed time: {video_timer.elapsed_time_str}s)')


    def save(self):
        self.__clean_results()
        if self.detailed_bout_data and len(self.detailed_df) > 0:
            self.detailed_df.to_csv(self.detailed_bout_data_save_path)
            print(f"Detailed ROI data saved at {self.detailed_bout_data_save_path}...")
        self.results.to_csv(self.save_path)
        self.timer.stop_timer()
        stdout_success(f'ROI statistics saved at {self.save_path}', elapsed_time=self.timer.elapsed_time_str)


# test = ROITimebinAnalyzer(config_path=r"C:\troubleshooting\mitra\project_folder\project_config.ini",
#                           bin_size=500,
#                           body_parts=['Nose'],
#                           detailed_bout_data=True,
#                           calculate_distances=True,
#                           transpose=False,
#                           outside_rois=True,
#                           verbose=True)
# test.run()
# test.save()
# #



# test = ROITimebinCalculator(config_path=r"C:\troubleshooting\spontenous_alternation\project_folder\project_config.ini",
#                             bin_length=0.5,
#                             body_parts=['nose'],
#                             threshold=0.00,
#                             movement=True)
# test.run()
# test.save()


# test = ROITimebinCalculator(config_path=r"C:\troubleshooting\ROI_movement_test\project_folder\project_config.ini",
#                             bin_length=0.5,
#                             body_parts=['Head'],
#                             threshold=0.00,
#                             movement=True)
# test.run()
# test.save()

# test = ROITimebinCalculator(config_path=r"/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini",
#                             bin_length=1,
#                             body_parts=['Nose_1'],
#                             threshold=0.00,
#                             movement=True)
# test.run()
# test.save()



__author__ = "Simon Nilsson"

import os
import threading
from copy import deepcopy
from tkinter import *
from typing import Union

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.path_plotter import PathPlotterSingleCore
from simba.plotting.path_plotter_mp import PathPlotterMulticore
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, DropDownMenu,
                                        Entry_Box, SimbaButton, SimbaCheckbox,
                                        SimBADropDown)
from simba.utils.checks import (check_if_string_value_is_valid_video_timestamp,
                                check_if_valid_rgb_str, check_int,
                                check_that_hhmmss_start_is_before_end)
from simba.utils.enums import Formats, Links
from simba.utils.errors import (FrameRangeError, NoFilesFoundError,
                                NoROIDataError)
from simba.utils.lookups import get_color_dict
from simba.utils.read_write import get_file_name_info_in_directory

AUTO = 'AUTO'
CUSTOM = 'CUSTOM'
ENTIRE_VIDEO = 'ENTIRE VIDEO'
VIDEO_STATIC_FRAME = 'Video - static frame'
VIDEO_MOVING_FRAME = 'Video - moving frames'
STYLE_WIDTH = "width"
STYLE_HEIGHT = "height"
STYLE_LINE_WIDTH = "line width"
STYLE_FONT_SIZE = "font size"
STYLE_FONT_THICKNESS = "font thickness"
STYLE_CIRCLE_SIZE = "circle size"
STYLE_MAX_LINES = "max lines"
STYLE_BG = 'bg'
STYLE_BG_OPACITY = 'bg_opacity'
COLOR = 'color'
SIZE = 'size'
START_TIME = 'start_time'
END_TIME = 'end_time'
BODY_PART = 'body_part'
ANIMAL_NAME = 'animal_name'

class PathPlotPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: Union[str, os.PathLike]):
        ConfigReader.__init__(self, config_path=config_path, read_video_info=False)
        self.machine_results_files = get_file_name_info_in_directory(directory=self.machine_results_dir, file_type=self.file_type)
        self.outlier_corrected_files = get_file_name_info_in_directory(directory=self.outlier_corrected_dir, file_type=self.file_type)
        self.files_found = list(set(list(self.machine_results_files.keys()) + list(self.outlier_corrected_files.keys())))
        if len(self.files_found) == 0:
            raise NoFilesFoundError(msg=f'No data files found inside the {self.outlier_corrected_dir} or the {self.machine_results_dir} directory', source=self.__class__.__name__)
        PopUpMixin.__init__(self, title="CREATE PATH PLOTS", size=(650, 850), icon='path_2')
        self.resolution_options = deepcopy(self.resolutions)
        self.resolution_options.insert(0, AUTO)
        self.bg_clr_options = deepcopy(list(self.colors_dict.keys()))
        self.animal_trace_clrs = deepcopy(list(self.colors_dict.keys()))
        self.bg_clr_options.extend((VIDEO_STATIC_FRAME, VIDEO_MOVING_FRAME))
        self.animal_trace_clrs.append(CUSTOM)
        self.max_prior_lines_options = list(range(1, 61, 1))
        self.max_prior_lines_options.insert(0, ENTIRE_VIDEO)
        self.bg_opacity_options = list(range(10, 110, 10))
        self.animal_cnt_options = list(range(1, self.animal_cnt + 1))
        self.line_width_options = list(range(1, 26, 1))
        self.line_width_options.insert(0, AUTO)
        self.font_size_options = list(range(1, 26, 1))
        self.font_size_options.insert(0, AUTO)
        self.font_thickness_options = list(range(1, 26, 1))
        self.font_thickness_options.insert(0, AUTO)
        self.circle_size_options = list(range(1, 26, 1))
        self.circle_size_options.insert(0, AUTO)

        self.custom_rgb_selections = {}

        self.style_settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="STYLE SETTINGS", icon_name='style', icon_link=Links.PATH_PLOTS.value)
        self.max_prior_lines_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.max_prior_lines_options, label='MAX PRIOR LINES (S): ', label_width=35, dropdown_width=30, value=ENTIRE_VIDEO)
        self.resolution_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.resolution_options, label='RESOLUTION: ', label_width=35, dropdown_width=30, value=AUTO)
        self.bg_clr_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.bg_clr_options, label='BACKGROUND: ', label_width=35, dropdown_width=30, value='White', command=self.__activate_settings)
        self.line_width_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.line_width_options, label='LINE WIDTH: ', label_width=35, dropdown_width=30, value=AUTO)
        self.font_size_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.font_size_options, label='FONT SIZE: ', label_width=35, dropdown_width=30, value=AUTO)
        self.font_thickness_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.font_thickness_options, label='FONT THICKNESS: ', label_width=35, dropdown_width=30, value=AUTO)
        self.bg_opacity_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.bg_opacity_options, label='BACKGROUND OPACITY (%): ', label_width=35, dropdown_width=30, value=100)
        self.circle_size_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.circle_size_options, label='CIRCLE SIZE: ', label_width=35, dropdown_width=30, value=AUTO)
        self.body_parts_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="CHOOSE BODY-PARTS", icon_name='pose', icon_link=Links.PATH_PLOTS.value)
        self.number_of_animals_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.animal_cnt_options, label="# ANIMALS:", label_width=35, dropdown_width=30, value=self.animal_cnt_options[0], command=self.populate_body_parts_menu)

        self.style_settings_frm.grid(row=0, sticky=NW)
        self.resolution_dropdown.grid(row=0, sticky=NW)
        self.max_prior_lines_dropdown.grid(row=2, sticky=NW)
        self.line_width_dropdown.grid(row=4, sticky=NW)
        self.circle_size_dropdown.grid(row=5, sticky=NW)
        self.font_size_dropdown.grid(row=6, sticky=NW)
        self.font_thickness_dropdown.grid(row=7, sticky=NW)
        self.bg_clr_dropdown.grid(row=8, sticky=NW)
        self.bg_opacity_dropdown.grid(row=9, sticky=NW)

        self.video_slicing_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="SEGMENTS", icon_name='clip', icon_link=Links.PATH_PLOTS.value)
        self.slice_var = BooleanVar(value=False)
        self.video_start_time_entry = Entry_Box(self.video_slicing_frm, "START TIME:", "35", entry_box_width=30, value="00:00:00", status=DISABLED)
        self.video_end_time_entry = Entry_Box(self.video_slicing_frm, "END TIME:", "35", entry_box_width=30, value="00:00:00", status=DISABLED)
        self.slice_cb = Checkbutton(self.video_slicing_frm, text="Plot ONLY defined time-segment", font=Formats.FONT_REGULAR_BOLD.value, variable=self.slice_var, command=lambda: self.enable_entrybox_from_checkbox(check_box_var=self.slice_var, entry_boxes=[self.video_start_time_entry, self.video_end_time_entry]))
        self.video_slicing_frm.grid(row=1, sticky=NW)
        self.slice_cb.grid(row=0, column=0, sticky=NW)
        self.video_start_time_entry.grid(row=1, column=0, sticky=NW)
        self.video_end_time_entry.grid(row=2, column=0, sticky=NW)

        self.clf_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="CLASSIFICATION VISUALIZATION", icon_name='forest', icon_link=Links.PATH_PLOTS.value)
        self.include_clf_locations_var = BooleanVar(value=False)
        self.include_clf_locations_cb = Checkbutton(self.clf_frm, text="INCLUDE CLASSIFICATION LOCATIONS", font=Formats.FONT_REGULAR.value, variable=self.include_clf_locations_var, command=self.populate_clf_location_data)
        self.include_clf_locations_cb.grid(row=0, sticky=NW)
        self.populate_clf_location_data()
        self.clf_frm.grid(row=2, sticky=NW)

        self.roi_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="ROI VISUALIZATION", icon_name='shapes_small', icon_link=Links.PATH_PLOTS.value)
        roi_cb, self.roi_var = SimbaCheckbox(parent=self.roi_frm, txt='INCLUDE ROIs', txt_img='roi', val=False)

        self.populate_body_parts_menu(self.animal_cnt_options[0])

        self.settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="VISUALIZATION SETTINGS", icon_name='eye', icon_link=Links.PATH_PLOTS.value)

        path_frames_cb, self.path_frames_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE FRAMES', txt_img='frames', val=False)
        path_videos_cb, self.path_videos_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE VIDEOS', txt_img='video', val=False)
        path_last_frm_cb, self.path_last_frm_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE LAST FRAME', txt_img='finish', val=True)
        self.include_animal_names_cb, self.include_animal_names_var = SimbaCheckbox(parent=self.settings_frm, txt='INCLUDE ANIMAL NAMES', txt_img='id_card')
        self.core_cnt_dropdown = SimBADropDown(parent=self.settings_frm, dropdown_options=list(range(1, self.cpu_cnt+1)), label='CPU CORE COUNT: ', label_width=35, dropdown_width=30, value=int(self.cpu_cnt/2))

        self.run_frm = LabelFrame(self.main_frm, text="RUN", font=Formats.FONT_HEADER.value, pady=5, padx=5, fg="black")
        self.run_single_video_frm = LabelFrame(self.run_frm, text="SINGLE VIDEO", font=Formats.FONT_HEADER.value, pady=5, padx=5, fg="black")
        self.run_single_video_btn = SimbaButton(parent=self.run_single_video_frm, txt='CREATE SINGLE VIDEO', img='rocket', txt_clr='blue', font=Formats.FONT_REGULAR.value, cmd=self._run, cmd_kwargs={'multiple_videos': False})

        self.single_video_dropdown = DropDownMenu( self.run_single_video_frm, "Video:", self.files_found, "12")
        self.single_video_dropdown.setChoices(self.files_found[0])
        self.run_multiple_videos = LabelFrame( self.run_frm, text="MULTIPLE VIDEO", font=Formats.FONT_HEADER.value, pady=5, padx=5, fg="black")
        self.run_multiple_video_btn = SimbaButton(parent=self.run_multiple_videos, txt=f"Create multiple videos ({len(self.files_found)} video(s) found)", font=Formats.FONT_REGULAR.value, img='rocket', txt_clr='blue', cmd=self._run, cmd_kwargs={'multiple_videos': True})



        self.roi_frm.grid(row=3, sticky=NW)
        roi_cb.grid(row=0, sticky=NW)

        self.body_parts_frm.grid(row=4, sticky=NW)
        self.number_of_animals_dropdown.grid(row=0, sticky=NW)

        self.settings_frm.grid(row=5, sticky=NW)
        self.core_cnt_dropdown.grid(row=0, column=0, sticky=NW)
        path_frames_cb.grid(row=1, sticky=NW)
        path_videos_cb.grid(row=2, sticky=NW)
        path_last_frm_cb.grid(row=3, sticky=NW)
        self.include_animal_names_cb.grid(row=4, sticky=NW)


        self.run_frm.grid(row=6, sticky=NW)
        self.run_single_video_frm.grid(row=0, sticky=NW)
        self.run_single_video_btn.grid(row=0, column=0, sticky=NW)
        self.single_video_dropdown.grid(row=0, column=1, sticky=NW)

        self.run_multiple_videos.grid(row=1, sticky=NW)
        self.run_multiple_video_btn.grid(row=0, sticky=NW)

        self.main_frm.mainloop()

    def populate_body_parts_menu(self, choice):
        if hasattr(self, "bp_dropdowns"):
            for (k, v), (k2, v2) in zip(self.bp_dropdowns.items(), self.bp_colors.items()):
                self.bp_dropdowns[k].destroy()
                self.bp_colors[k].destroy()
        for k, v in self.custom_rgb_selections.items():
            v.destroy()
        self.custom_rgb_selections = {}

        self.bp_dropdowns, self.bp_colors = {}, {}
        self.bp_row_idx = []
        for animal_cnt in range(int(self.number_of_animals_dropdown.getChoices())):
            self.bp_dropdowns[animal_cnt] = DropDownMenu(self.body_parts_frm, "Body-part {}:".format(str(animal_cnt + 1)), self.body_parts_lst, "16")
            self.bp_dropdowns[animal_cnt].setChoices(self.body_parts_lst[animal_cnt])
            self.bp_dropdowns[animal_cnt].grid(row=animal_cnt + 1, column=0, sticky=NW)
            self.bp_colors[animal_cnt] = DropDownMenu(self.body_parts_frm, "", self.animal_trace_clrs, "2", com=lambda x, k=animal_cnt: self.__set_custom_clrs(choice=x, row=k))
            self.bp_colors[animal_cnt].setChoices(list(self.colors_dict.keys())[animal_cnt])
            self.bp_colors[animal_cnt].grid(row=animal_cnt + 1, column=1, sticky=NW)

    def __activate_settings(self, choice: str):
        if choice == VIDEO_STATIC_FRAME or choice == VIDEO_MOVING_FRAME:
            self.bg_opacity_dropdown.enable()
        else:
            self.bg_opacity_dropdown.disable()
        if choice == VIDEO_STATIC_FRAME:
            self.static_frm_index_eb = Entry_Box(self.style_settings_frm, "SELECT FRAME INDEX: ", labelwidth=20, entry_box_width=10, validation="numeric", value=1)
            self.static_frm_index_eb.grid(row=8, column=1, sticky=NW)
        else:
            if hasattr(self, "static_frm_index_eb"):
                self.static_frm_index_eb.destroy()

    def __set_custom_clrs(self, choice: str, row: int):
        if choice == CUSTOM:
            self.custom_rgb_selections[row] = Entry_Box(self.body_parts_frm, "RGB:", "5", entry_box_width=10)
            self.custom_rgb_selections[row].entry_set(val="255,0,0")
            self.custom_rgb_selections[row].grid(row=row + 1, column=3, sticky=NW)
        else:
            if row in self.custom_rgb_selections.keys():
                self.custom_rgb_selections[row].destroy()
                self.custom_rgb_selections.pop(row)

    def populate_clf_location_data(self):
        self.clf_name, self.clf_clr, self.clf_size = {}, {}, {}
        size_lst = list(range(1, 51))
        size_lst = ["Size: " + str(x) for x in size_lst]
        for clf_cnt, clf_name in enumerate(self.clf_names):
            self.clf_name[clf_cnt] = DropDownMenu(self.clf_frm, "Classifier {}:".format(str(clf_cnt + 1)), self.clf_names, "16")
            self.clf_name[clf_cnt].setChoices(self.clf_names[clf_cnt])
            self.clf_name[clf_cnt].grid(row=clf_cnt + 1, column=0, sticky=NW)

            self.clf_clr[clf_cnt] = DropDownMenu(self.clf_frm, "", list(self.colors_dict.keys()), "2")
            self.clf_clr[clf_cnt].setChoices(list(self.colors_dict.keys())[clf_cnt])
            self.clf_clr[clf_cnt].grid(row=clf_cnt + 1, column=1, sticky=NW)

            self.clf_size[clf_cnt] = DropDownMenu(self.clf_frm, "", size_lst, "2")
            self.clf_size[clf_cnt].setChoices(size_lst[15])
            self.clf_size[clf_cnt].grid(row=clf_cnt + 1, column=2, sticky=NW)

        self.enable_clf_location_settings()

    def enable_clf_location_settings(self):
        if self.include_clf_locations_var.get():
            for clf_cnt in self.clf_name.keys():
                self.clf_name[clf_cnt].enable(); self.clf_clr[clf_cnt].enable(); self.clf_size[clf_cnt].enable()
        else:
            for clf_cnt in self.clf_name.keys():
                self.clf_name[clf_cnt].disable(); self.clf_clr[clf_cnt].disable(); self.clf_size[clf_cnt].disable()

    def _get_animal_attr(self):
        animal_attr = {}
        for cnt, (key, value) in enumerate(self.bp_colors.items()):
            if cnt not in animal_attr.keys():
                animal_attr[cnt] = {}
            clr = value.getChoices()
            if clr == CUSTOM:
                clr = self.custom_rgb_selections[cnt].entry_get
                clr = check_if_valid_rgb_str(input=clr)
                animal_attr[cnt]["color"] = clr
            else:
                animal_attr[cnt]["color"] = get_color_dict()[value.getChoices()]

        for cnt, (key, value) in enumerate(self.bp_dropdowns.items()):
            if cnt not in animal_attr.keys():
                animal_attr[cnt] = {}
            animal_attr[cnt]["body_part"] = value.getChoices()
        return animal_attr

    def _get_slicing(self):
        if self.slice_var.get():
            check_if_string_value_is_valid_video_timestamp(value=self.video_start_time_entry.entry_get, name="Video slicing START TIME")
            check_if_string_value_is_valid_video_timestamp(value=self.video_end_time_entry.entry_get, name="Video slicing END TIME")
            if (self.video_start_time_entry.entry_get == self.video_end_time_entry.entry_get):
                raise FrameRangeError(msg="The sliced start and end times cannot be identical", source=self.__class__.__name__)
            check_that_hhmmss_start_is_before_end(start_time=self.video_start_time_entry.entry_get, end_time=self.video_end_time_entry.entry_get, name="SLICE TIME STAMPS")
            return {"start_time": self.video_start_time_entry.entry_get, "end_time": self.video_end_time_entry.entry_get}
        else:
            return None


    def _get_clf_attr(self, multiple_videos):
        if multiple_videos:
            if len(self.machine_results_paths) == 0:
                raise NoFilesFoundError(msg=f'No DATA found in {self.machine_results_dir} directory. Un-check the classifier location checkbox, OR make sure the folder contains classification data.')
            else:
                data_paths = list(self.machine_results_files.values())
        else:
            if self.single_video_dropdown.getChoices() not in self.machine_results_files.keys():
                raise NoFilesFoundError(msg=f'No DATA found for video in {self.single_video_dropdown.getChoices()} in directory {self.machine_results_dir}. Un-check the classifier location checkbox, OR make sure the folder contains classification data for the video.')
            else:
                data_paths = [self.machine_results_files[self.single_video_dropdown.getChoices()]]
        clf_attr = {}
        for cnt, (key, value) in enumerate(self.clf_name.items()):
            clf_attr[value.getChoices()] = {}
            clf_attr[value.getChoices()]["color"] = get_color_dict()[self.clf_clr[cnt].getChoices()]
            size = "".join(filter(str.isdigit, self.clf_size[cnt].getChoices()))
            clf_attr[value.getChoices()]["size"] = int(size)
        return clf_attr, data_paths

    def _run(self, multiple_videos: bool):
        resolution = self.resolution_dropdown.getChoices()
        line_width = self.line_width_dropdown.get_value()
        circle_size = self.circle_size_dropdown.get_value()
        font_size = self.font_size_dropdown.get_value()
        path_thickness = self.font_thickness_dropdown.get_value()
        bg = self.bg_clr_dropdown.getChoices()
        max_lines = self.max_prior_lines_dropdown.get_value()
        include_clf = self.include_clf_locations_var.get()
        roi = self.roi_var.get()
        core_cnt = int(self.core_cnt_dropdown.get_value())
        print_animal_names = self.include_animal_names_var.get()
        clf_attr = None

        w, h = (None, None) if resolution == AUTO else tuple(map(int, resolution.split("×")))
        line_width = None if line_width == AUTO else int(line_width)
        circle_size = None if circle_size == AUTO else int(circle_size)
        font_size = None if font_size == AUTO else int(font_size)
        path_thickness = None if path_thickness == AUTO else int(path_thickness)
        max_prior_lines = None if max_lines == ENTIRE_VIDEO else int(max_lines)

        if bg == VIDEO_STATIC_FRAME:
            check_int(name="Static frame index", value=self.static_frm_index_eb.entry_get, min_value=0)
            bg = int(self.static_frm_index_eb.entry_get)
            bg_opacity = int(self.bg_opacity_dropdown.get_value())
        elif bg == VIDEO_MOVING_FRAME:
            bg = 'video'
            bg_opacity = int(self.bg_opacity_dropdown.get_value())
        else:
            bg = get_color_dict()[bg]
            bg_opacity = 100
        animal_attr = self._get_animal_attr()
        slicing = self._get_slicing()
        if include_clf:
            clf_attr, data_paths = self._get_clf_attr(multiple_videos=multiple_videos)
        else:
            if multiple_videos:
                data_paths = list(self.outlier_corrected_files.values())
            else:
                data_paths = [self.outlier_corrected_files[self.single_video_dropdown.getChoices()]]

        if roi:
            if not os.path.isfile(self.roi_coordinates_path):
                raise NoROIDataError(msg=f'No SimBA ROI project data found. Expected at path {self.roi_coordinates_path}', source=self.__class__.__name__)

        style_attr = {STYLE_WIDTH: w,
                      STYLE_HEIGHT: h,
                      STYLE_MAX_LINES: max_prior_lines,
                      STYLE_LINE_WIDTH: line_width,
                      STYLE_FONT_SIZE: font_size,
                      STYLE_FONT_THICKNESS: path_thickness,
                      STYLE_CIRCLE_SIZE: circle_size,
                      STYLE_BG: bg,
                      STYLE_BG_OPACITY: bg_opacity}

        if core_cnt == 1:
            path_plotter = PathPlotterSingleCore(config_path=self.config_path,
                                                 frame_setting=self.path_frames_var.get(),
                                                 video_setting=self.path_videos_var.get(),
                                                 last_frame=self.path_last_frm_var.get(),
                                                 data_paths=data_paths,
                                                 style_attr=style_attr,
                                                 print_animal_names=print_animal_names,
                                                 animal_attr=animal_attr,
                                                 clf_attr=clf_attr,
                                                 slicing=slicing,
                                                 roi=roi)
        else:
            path_plotter = PathPlotterMulticore(config_path=self.config_path,
                                                frame_setting=self.path_frames_var.get(),
                                                video_setting=self.path_videos_var.get(),
                                                last_frame=self.path_last_frm_var.get(),
                                                data_paths=data_paths,
                                                style_attr=style_attr,
                                                print_animal_names=print_animal_names,
                                                animal_attr=animal_attr,
                                                clf_attr=clf_attr,
                                                core_cnt=core_cnt,
                                                slicing=slicing,
                                                roi=self.roi_var.get())

        threading.Thread(target=path_plotter.run()).start()



#_ = PathPlotPopUp(config_path=r"C:\troubleshooting\RAT_NOR\project_folder\project_config.ini")

#_ = PathPlotPopUp(config_path=r"C:\troubleshooting\mitra\project_folder\project_config.ini")

#_ = PathPlotPopUp(config_path=r"/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini")

# _ = PathPlotPopUp(config_path='/Users/simon/Desktop/envs/simba/troubleshooting/beepboop174/project_folder/project_config.ini')

# _ = PathPlotPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')

# _ = PathPlotPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/dorian_2/project_folder/project_config.ini')



import os
from tkinter import *
from typing import Union

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, SimbaButton,
                                        SimBADropDown)
from simba.utils.checks import check_float
from simba.utils.enums import ConfigKey, Dtypes, Formats
from simba.utils.printing import stdout_success
from simba.utils.read_write import read_config_entry

WINDOW_SIZE_OPTIONS = [round(x * 0.05, 2) for x in range(21)]

class SetMinMaxDrawWindowSize(ConfigReader, PopUpMixin):


    def __init__(self,
                 config_path: Union[str, os.PathLike]):

        ConfigReader.__init__(self, config_path=config_path, read_video_info=False, create_logger=False)
        PopUpMixin.__init__(self, title="SET DRAWING WINDOW SIZE", size=(720, 960), icon='monitor')

        min_draw_display_ratio_h = read_config_entry(config=self.config, section=ConfigKey.DISPLAY_SETTINGS.value, option=ConfigKey.MIN_ROI_DISPLAY_HEIGHT.value, default_value=0.60, data_type=Dtypes.FLOAT.value)
        min_draw_display_ratio_w = read_config_entry(config=self.config, section=ConfigKey.DISPLAY_SETTINGS.value, option=ConfigKey.MIN_ROI_DISPLAY_WIDTH.value, default_value=0.30, data_type=Dtypes.FLOAT.value)
        max_draw_display_ratio_h = read_config_entry(config=self.config, section=ConfigKey.DISPLAY_SETTINGS.value, option=ConfigKey.MAX_ROI_DISPLAY_HEIGHT.value, default_value=0.75, data_type=Dtypes.FLOAT.value)
        max_draw_display_ratio_w = read_config_entry(config=self.config, section=ConfigKey.DISPLAY_SETTINGS.value, option=ConfigKey.MAX_ROI_DISPLAY_WIDTH.value, default_value=0.50, data_type=Dtypes.FLOAT.value)

        for i in [min_draw_display_ratio_h, min_draw_display_ratio_w, max_draw_display_ratio_h, max_draw_display_ratio_w]:
            check_float(name=f'{self.__class__.__name__} size', value=i, max_value=1, min_value=0, raise_error=True)

        settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="DISPLAY SETTINGS - ROI DRAW WINDOW", font=Formats.FONT_HEADER.value, padx=5, pady=5, icon_name='settings', relief='solid')
        self.max_width_ratio_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=WINDOW_SIZE_OPTIONS, label="MAX DRAW DISPLAY RATIO WIDTH: ", label_width=35, dropdown_width=35, value=max_draw_display_ratio_w)
        self.max_height_ratio_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=WINDOW_SIZE_OPTIONS, label="HEIGHT: ", label_width=10, dropdown_width=10, value=max_draw_display_ratio_h)
        self.min_width_ratio_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=WINDOW_SIZE_OPTIONS, label="MIN DRAW DISPLAY RATIO WIDTH: ", label_width=35, dropdown_width=35, value=min_draw_display_ratio_w)
        self.min_height_ratio_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=WINDOW_SIZE_OPTIONS, label="HEIGHT: ", label_width=10, dropdown_width=10, value=min_draw_display_ratio_h)

        settings_frm.grid(row=0, column=0, sticky=NW)
        self.max_width_ratio_dropdown.grid(row=0, column=0, sticky=NW, pady=5)
        self.max_height_ratio_dropdown.grid(row=0, column=1, sticky=NW, pady=5)
        self.min_width_ratio_dropdown.grid(row=1, column=0, sticky=NW, pady=5)
        self.min_height_ratio_dropdown.grid(row=1, column=1, sticky=NW, pady=5)


        set_btn = SimbaButton(parent=self.main_frm, txt='SET', img='rocket', cmd=self.run)
        set_btn.grid(row=1, column=0, sticky=NW)
        #self.main_frm.mainloop()

    def run(self):
        max_width = str(self.max_width_ratio_dropdown.get_value())
        max_height = str(self.max_height_ratio_dropdown.get_value())
        min_width = str(self.min_width_ratio_dropdown.get_value())
        min_height = str(self.min_height_ratio_dropdown.get_value())

        if not self.config.has_section(ConfigKey.DISPLAY_SETTINGS.value): self.config.add_section(ConfigKey.DISPLAY_SETTINGS.value)
        self.config[ConfigKey.DISPLAY_SETTINGS.value][ConfigKey.MAX_ROI_DISPLAY_HEIGHT.value] = max_height
        self.config[ConfigKey.DISPLAY_SETTINGS.value][ConfigKey.MAX_ROI_DISPLAY_WIDTH.value] = max_width
        self.config[ConfigKey.DISPLAY_SETTINGS.value][ConfigKey.MIN_ROI_DISPLAY_WIDTH.value] = min_width
        self.config[ConfigKey.DISPLAY_SETTINGS.value][ConfigKey.MIN_ROI_DISPLAY_HEIGHT.value] = min_height

        with open(self.config_path, "w") as file:
            self.config.write(file)

        stdout_success(msg='DISPLAY SETTINGS SAVED!')




#SetMinMaxDrawWindowSize(config_path=r"C:\troubleshooting\open_field_below\project_folder\project_config.ini")